@@ -27,6 +27,9 @@
 #include "htmltypes.h"
 
 void  html_engine_copy                    (HTMLEngine  *e);
+void  html_engine_copy_object             (HTMLEngine  *e,
+					   HTMLObject **o,
+					   guint       *len);
 void  html_engine_cut                     (HTMLEngine  *e);
 void  html_engine_cut_line                (HTMLEngine  *e);
 void  html_engine_delete                  (HTMLEngine  *e);
@@ -53,6 +56,7 @@ void  html_engine_insert_empty_paragraph  (HTMLEngine  *e);
 void  html_engine_delete_n                (HTMLEngine  *e,
 					   guint        len,
 					   gboolean     forward);
+void  html_engine_delete_container        (HTMLEngine  *e);
 void  html_engine_set_insertion_link      (HTMLEngine  *e,
 					   const gchar *url,
 					   const gchar *target);
