@@ -2,6 +2,7 @@
 /*  This file is part of the GtkHTML library.
 
     Copyright (C) 1999, 2000 Helix Code, Inc.
+    Copyright (C) 2001 Ximian, Inc.
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -18,7 +19,8 @@
     the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
 
-    Author: Ettore Perazzoli <ettore@helixcode.com>
+    Authors: Ettore Perazzoli <ettore@helixcode.com>
+             Radek Doulik     <rodo@ximian.com>
 */
 
 
@@ -33,6 +35,7 @@
 #include "htmlclueflow.h"
 #include "htmlcursor.h"
 #include "htmllinktext.h"
+#include "htmltable.h"
 #include "htmltext.h"
 #include "htmltextslave.h"
 #include "htmlimage.h"
@@ -40,11 +43,13 @@
 #include "htmlselection.h"
 #include "htmlundo.h"
 
+#include "htmlengine-edit.h"
 #include "htmlengine-edit-cut-and-paste.h"
 #include "htmlengine-edit-cursor.h"
 #include "htmlengine-edit-movement.h"
 #include "htmlengine-edit-selection-updater.h"
-#include "htmlengine-edit.h"
+#include "htmlengine-edit-table.h"
+#include "htmlengine-edit-tablecell.h"
 
 
 void
@@ -203,7 +208,8 @@ html_engine_spell_check_range (HTMLEngine *e, HTMLCursor *begin, HTMLCursor *end
 
 	e->need_spell_check = FALSE;
 
-	if (!e->widget->editor_api || !GTK_HTML_CLASS (GTK_OBJECT (e->widget)->klass)->properties->live_spell_check)
+	if (!e->widget->editor_api || !GTK_HTML_CLASS (GTK_OBJECT (e->widget)->klass)->properties->live_spell_check
+	    || !begin->object->parent)
 		return;
 
 	begin = html_cursor_dup (begin);
@@ -217,7 +223,7 @@ html_engine_spell_check_range (HTMLEngine *e, HTMLCursor *begin, HTMLCursor *end
 	i = html_interval_new_from_cursor (begin, end);
 	if (begin->object->parent != end->object->parent)
 		html_interval_forall (i, e, spell_check_object, i);
-	else
+	else if (HTML_IS_CLUEFLOW (begin->object->parent))
 		html_clueflow_spell_check (HTML_CLUEFLOW (begin->object->parent), e, i);
 	html_interval_destroy (i);
 	html_cursor_destroy (begin);
@@ -225,10 +231,10 @@ html_engine_spell_check_range (HTMLEngine *e, HTMLCursor *begin, HTMLCursor *end
 }
 
 gboolean
-html_is_in_word (unicode_char_t uc)
+html_is_in_word (gunichar uc)
 {
-	/* printf ("test %d %c => %d\n", uc, uc, unicode_isalnum (uc) || uc == '\''); */
-	return unicode_isalpha (uc) || uc == '\'';
+	/* printf ("test %d %c => %d\n", uc, uc, g_unichar_isalnum (uc) || uc == '\''); */
+	return g_unichar_isalpha (uc) || uc == '\'';
 }
 
 void
@@ -249,10 +255,44 @@ html_engine_select_line_editable (HTMLEngine *e)
 	html_engine_end_of_line (e);
 }
 
+void
+html_engine_select_paragraph_editable (HTMLEngine *e)
+{
+	html_engine_beginning_of_paragraph (e);
+	html_engine_set_mark (e);
+	html_engine_end_of_paragraph (e);
+}
+
+void
+html_engine_select_paragraph_extended (HTMLEngine *e)
+{
+	gboolean fw;
+
+	html_engine_hide_cursor (e);
+	html_engine_beginning_of_paragraph (e);
+	fw = html_cursor_backward (e->cursor, e);
+	html_engine_set_mark (e);
+	if (fw)
+		html_cursor_forward (e->cursor, e);
+	html_engine_end_of_paragraph (e);
+	html_cursor_forward (e->cursor, e);
+	html_engine_show_cursor (e);
+
+	html_engine_update_selection_if_necessary (e);
+}
+
+void
+html_engine_select_all_editable (HTMLEngine *e)
+{
+	html_engine_beginning_of_document (e);
+	html_engine_set_mark (e);
+	html_engine_end_of_document (e);
+}
+
 struct SetData {
 	HTMLType      object_type;
 	const gchar  *key;
-	gpointer      value;
+	const gchar  *value;
 };
 
 static void
@@ -267,7 +307,7 @@ set_data (HTMLObject *o, HTMLEngine *e, gpointer p)
 }
 
 void
-html_engine_set_data_by_type (HTMLEngine *e, HTMLType object_type, const gchar *key, gpointer value)
+html_engine_set_data_by_type (HTMLEngine *e, HTMLType object_type, const gchar *key, const gchar * value)
 {
 	struct SetData *data = g_new (struct SetData, 1);
 
@@ -330,18 +370,26 @@ html_engine_get_indent (HTMLEngine *e)
 
 #define LINE_LEN 71
 
+static inline guint
+inc_line_offset (guint line_offset, gunichar uc)
+{
+	return uc == '\t'
+		? line_offset + 8 - (line_offset % 8)
+		: line_offset + 1;
+}
+
 static guint
 try_break_this_line (HTMLEngine *e, guint line_offset, guint last_space)
 {
 	HTMLObject *flow;
-	unicode_char_t uc;
+	gunichar uc;
 
 	flow = e->cursor->object->parent;
 
 	while (html_cursor_forward (e->cursor, e) && e->cursor->object->parent == flow) {
-		line_offset ++;
 		uc = html_cursor_get_current_char (e->cursor);
-		if (uc == ' ')
+		line_offset = inc_line_offset (line_offset, uc);
+		if (uc == ' ' || uc == '\t')
 			last_space = line_offset;
 		if (uc && line_offset >= LINE_LEN) {
 			if (last_space) {
@@ -350,13 +398,13 @@ try_break_this_line (HTMLEngine *e, guint line_offset, guint last_space)
 			} else {
 				/* go to end of word */
 				while (html_cursor_forward (e->cursor, e)) {
-					line_offset ++;
+					line_offset = inc_line_offset (line_offset, uc);
 					uc = html_cursor_get_current_char (e->cursor);
-					if (uc == ' ' || !uc)
+					if (uc == ' ' || uc == '\t' || !uc)
 						break;
 				}
 			}
-			if (uc == ' ') {
+			if (uc == ' ' || uc == '\t') {
 				html_engine_insert_empty_paragraph (e);
 				html_engine_delete_n (e, 1, TRUE);
 
@@ -415,8 +463,10 @@ html_engine_indent_pre_paragraph (HTMLEngine *e)
 		    && e->cursor->offset == 0 && html_object_get_length (e->cursor->object)
 		    && html_object_prev_not_slave (e->cursor->object) == NULL) {
 			if (line_offset < LINE_LEN - 1) {
+				gunichar prev;
 				html_engine_delete_n (e, 1, FALSE);
-				if (' ' != html_cursor_get_prev_char (e->cursor)) {
+				prev = html_cursor_get_prev_char (e->cursor);
+				if (prev != ' ' && prev != '\t') {
 					html_engine_insert_text (e, " ", 1);
 					line_offset ++;
 				} else if (position > e->cursor->position)
@@ -442,7 +492,7 @@ html_engine_indent_pre_line (HTMLEngine *e)
 	guint line_offset;
 	guint last_space;
 	HTMLObject *flow;
-	unicode_char_t uc;
+	gunichar uc;
 
 	g_assert (e->cursor->object);
 	if (HTML_OBJECT_TYPE (e->cursor->object->parent) != HTML_TYPE_CLUEFLOW
@@ -463,10 +513,10 @@ html_engine_indent_pre_line (HTMLEngine *e)
 	flow = e->cursor->object->parent;
 	while (html_cursor_forward (e->cursor, e) && e->cursor->object->parent == flow) {
 		uc = html_cursor_get_current_char (e->cursor);
-		line_offset++;
+		line_offset = inc_line_offset (line_offset, uc);
 
-		if (uc == ' ') {
-				last_space = line_offset;
+		if (uc == ' ' || uc == '\t') {
+			last_space = line_offset;
 		}
 		
 		if (line_offset >= LINE_LEN) {
@@ -497,7 +547,7 @@ html_engine_fill_pre_line (HTMLEngine *e)
 	guint line_offset;
 	guint last_space;
 	HTMLObject *flow;
-	unicode_char_t uc;
+	gunichar uc;
 
 	g_assert (e->cursor->object);
 	position = e->cursor->position;
@@ -515,9 +565,9 @@ html_engine_fill_pre_line (HTMLEngine *e)
 	flow = e->cursor->object->parent;
 	while (html_cursor_forward (e->cursor, e) && (e->cursor->position < position - 1)) {
 		uc = html_cursor_get_current_char (e->cursor);
-		line_offset++;
+		line_offset = inc_line_offset (line_offset, uc);
 		
-		if (uc == ' ') {
+		if (uc == ' ' || uc == '\t') {
 			last_space = line_offset;
 		}
 		
@@ -574,3 +624,51 @@ html_engine_break_and_fill_line (HTMLEngine *e)
 	html_engine_thaw (e);
 	html_undo_level_end (e->undo);
 }
+
+gboolean
+html_engine_next_cell (HTMLEngine *e, gboolean create)
+{
+	HTMLTableCell *cell, *current_cell;
+
+	cell = html_engine_get_table_cell (e);
+	if (cell) {
+		html_engine_hide_cursor (e);
+		do {
+			html_cursor_end_of_line (e->cursor, e);
+			html_cursor_forward (e->cursor, e);
+			current_cell = html_engine_get_table_cell (e);
+		} while (current_cell == cell);
+			
+		if (create && HTML_IS_TABLE (e->cursor->object)) {
+			html_cursor_backward (e->cursor, e);
+			html_engine_insert_table_row (e, TRUE);
+		}
+		html_engine_show_cursor (e);
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+gboolean
+html_engine_prev_cell (HTMLEngine *e)
+{
+	HTMLTableCell *cell, *current_cell;
+
+	cell = html_engine_get_table_cell (e);
+	if (cell) {
+		html_engine_hide_cursor (e);
+		do {
+			html_cursor_beginning_of_line (e->cursor, e);
+			html_cursor_backward (e->cursor, e);
+			current_cell = html_engine_get_table_cell (e);
+		} while (current_cell == cell);
+
+		html_engine_show_cursor (e);
+
+		return TRUE;
+	}
+
+	return FALSE;
+}
