@@ -26,6 +26,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <regex.h>
+#include <gal/widgets/e-unicode.h>
 
 #include "htmltext.h"
 #include "htmlcolor.h"
@@ -52,6 +53,7 @@ static void         spell_error_destroy     (SpellError *se);
 static void         move_spell_errors       (GList *spell_errors, guint offset, gint delta);
 static GList *      remove_spell_errors     (GList *spell_errors, guint offset, guint len);
 static guint        get_words               (const gchar *s);
+static void         remove_text_slaves      (HTMLObject *self);
 
 /* static void
 debug_spell_errors (GList *se)
@@ -174,7 +176,7 @@ copy (HTMLObject *s,
 	dest->word_width = NULL;
 }
 
-static void
+/* static void
 debug_word_width (HTMLText *t)
 {
 	guint i;
@@ -198,7 +200,7 @@ word_get_position (HTMLText *text, guint off, guint *word_out, guint *left_out,
 		ls    = s;
 		loff  = coff;
 		s     = strchr (s, ' ');
-		coff += s ? unicode_index_to_offset (ls, s - ls) : unicode_strlen (ls, -1);
+		coff += s ? g_utf8_pointer_to_offset (ls, s) : g_utf8_strlen (ls, -1);
 		(*word_out) ++;
 		if (s)
 			s ++;
@@ -207,8 +209,8 @@ word_get_position (HTMLText *text, guint off, guint *word_out, guint *left_out,
 	*left_out  = off - loff;
 	*right_out = coff - off;
 
-	/* printf ("get position w: %d l: %d r: %d\n", *word_out, *left_out, *right_out); */
-}
+	printf ("get position w: %d l: %d r: %d\n", *word_out, *left_out, *right_out);
+} */
 
 void
 html_text_clear_word_width (HTMLText *text)
@@ -218,91 +220,6 @@ html_text_clear_word_width (HTMLText *text)
 	text->words = 0;
 }
 
-static void
-merge_word_width (HTMLText *t1, HTMLText *t2, HTMLPainter *p)
-{
-	guint len, i, words;
-
-	/* printf ("before merge '%s' '%s'\n", t1->text, t2->text);
-	   debug_word_width (t1);
-	   debug_word_width (t2); */
-
-	if (!t1->word_width)
-		return;
-
-	len = strlen (t1->text);
-	if (((len && t1->text [len - 1] == ' ')
-	     || (len > 1 && (guchar) t1->text [len - 1] == 0xa0 && (guchar) t1->text [len - 1] == 0xc2))
-	    && t2->text [0] == ' ') {
-		html_text_clear_word_width (t1);
-		return; /* we don't want do merge as convert_nbsp will 100% happen */
-	}
-
-	if (!t2->word_width)
-		html_text_request_word_width (t2, p);
-
-	words          = t1->words;
-	t1->words      = words + t2->words - 1;
-	t1->word_width = g_renew (guint, t1->word_width, t1->words);
-
-	for (i = 1; i < t2->words; i ++)
-		t1->word_width [words + i - 1] = t2->word_width [i] + t1->word_width [words - 1];
-	t1->word_width [words - 1] += t2->word_width [0];
-
-	/* printf ("after merge '%s%s'\n", t1->text, t2->text);
-	   debug_word_width (t1); */
-}
-
-static void
-split_word_width (HTMLText *s, HTMLText *d, HTMLPainter *p, gint offset)
-{
-	gchar *str;
-	guint words, i;
-	gboolean in_middle;
-
-	html_text_clear_word_width (d);
-	if (!s->word_width)
-		return;
-
-	/* printf ("before split '%s%s'\n", s->text, d->text);
-	   debug_word_width (s);
-	   debug_word_width (d); */
-
-	words     = get_words (s->text);
-	in_middle = d->text [0] == ' ' ? FALSE : TRUE;
-
-	/* fill d */
-	d->words      = s->words - words + 1;
-	d->word_width = g_new (guint, d->words);
-	if (in_middle) {
-		str = strchr (d->text, ' ');
-		d->word_width [0] = html_painter_calc_text_width (p, d->text, str
-								  ? unicode_index_to_offset (d->text, str - d->text)
-								  : d->text_len, html_text_get_font_style (s), s->face);
-	} else
-		d->word_width [0] = 0;
-	for (i = 1; i < d->words; i ++)
-		d->word_width [i] = s->word_width [words + i - 1] - s->word_width [words - 1] + d->word_width [0];
-
-	/* fill s */
-	if (s->words != words) {
-		s->words = words;
-		s->word_width = g_renew (guint, s->word_width, s->words);
-	}
-
-	if (in_middle) {
-		str = strrchr (s->text, ' ');
-		if (!str)
-			str = s->text;
-		s->word_width [s->words - 1] = html_painter_calc_text_width (p, str, unicode_strlen (str, -1),
-									     html_text_get_font_style (s), s->face)
-			+ (s->words > 1 ? s->word_width [s->words - 2] : 0);
-	}
-	/* printf ("after split '%s' '%s'\n", s->text, d->text);
-	   debug_word_width (s);
-	   debug_word_width (d); */
-}
-
 HTMLObject *
 html_text_op_copy_helper (HTMLText *text, GList *from, GList *to, guint *len, HTMLTextHelperFunc f)
 {
@@ -342,6 +259,7 @@ html_text_op_cut_helper (HTMLText *text, HTMLEngine *e, GList *from, GList *to,
 	/* printf ("before cut '%s'\n", text->text);
 	   debug_word_width (text); */
 
+	remove_text_slaves (HTML_OBJECT (text));
 	if (!html_object_could_remove_whole (HTML_OBJECT (text), from, to, left, right) || begin || end < text->text_len) {
 		gchar *nt, *tail;
 
@@ -371,7 +289,7 @@ html_text_op_cut_helper (HTMLText *text, HTMLEngine *e, GList *from, GList *to,
 	}
 
 	html_text_clear_word_width (text);
-	html_object_change_set (HTML_OBJECT (text), HTML_CHANGE_ALL);
+	html_object_change_set (HTML_OBJECT (text), HTML_CHANGE_ALL_CALC);
 
 	/* printf ("after cut '%s'\n", text->text);
 	   debug_word_width (text); */
@@ -410,9 +328,9 @@ object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList *left, GL
 	if (t1->font_style != t2->font_style || t1->color != t2->color)
 		return FALSE;
 
-	printf ("merge '%s' '%s'\n", t1->text, t2->text);
+	/* printf ("merge '%s' '%s'\n", t1->text, t2->text); */
 
-	merge_word_width (t1, t2, e->painter);
+	/* merge_word_width (t1, t2, e->painter); */
 
 	move_spell_errors (t2->spell_errors, 0, t1->text_len);
 	t1->spell_errors = g_list_concat (t1->spell_errors, t2->spell_errors);
@@ -423,11 +341,11 @@ object_merge (HTMLObject *self, HTMLObject *with, HTMLEngine *e, GList *left, GL
 	t1->text_len += t2->text_len;
 	g_free (to_free);
 	html_text_convert_nbsp (t1, TRUE);
-	html_object_change_set (self, HTML_CHANGE_ALL);
-
-	html_text_request_word_width (t1, e->painter);
-	printf ("merged '%s'\n", t1->text);
+	html_object_change_set (self, HTML_CHANGE_ALL_CALC);
 
+	html_text_clear_word_width (t1);
+	/* html_text_request_word_width (t1, e->painter); */
+	/* printf ("merged '%s'\n", t1->text); */
 	/* printf ("--- after merge\n");
 	   debug_spell_errors (t1->spell_errors);
 	   printf ("---\n"); */
@@ -460,14 +378,6 @@ object_split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, g
 	t2->text_len   -= offset;
 	if (!html_text_convert_nbsp (t2, FALSE))
 		t2->text = g_strdup (t2->text);
-	else {  /* take care of split in &nbsp; sequence */
-		guint len = strlen (t1->text);
-
-		if (t2->text [0] == ' '
-		    && ((len > 1 && (guchar) t1->text [len - 1] == 0xa0 && (guchar) t2->text [len - 2] == 0xc2)
-			|| (len && t1->text [len - 1] == ' ')))
-			html_text_clear_word_width (t1);
-	}
 	g_free (tt);
 
 	html_clue_append_after (HTML_CLUE (self->parent), dup, self);
@@ -498,12 +408,11 @@ object_split (HTMLObject *self, HTMLEngine *e, HTMLObject *child, gint offset, g
 	*left  = g_list_prepend (*left, self);
 	*right = g_list_prepend (*right, dup);
 
-	html_object_change_set (self, HTML_CHANGE_ALL);
-	html_object_change_set (dup,  HTML_CHANGE_ALL);
+	html_object_change_set (self, HTML_CHANGE_ALL_CALC);
+	html_object_change_set (dup,  HTML_CHANGE_ALL_CALC);
 
-	split_word_width (HTML_TEXT (self), HTML_TEXT (dup), e->painter, offset);
-	//html_text_clear_word_width (HTML_TEXT (self));
-	//html_text_clear_word_width (HTML_TEXT (dup));
+	html_text_clear_word_width (HTML_TEXT (self));
+	html_text_clear_word_width (HTML_TEXT (dup));
 
 	level--;
 	if (level)
@@ -524,6 +433,49 @@ calc_size (HTMLObject *self,
 	return FALSE;
 }
 
+gint
+html_text_text_line_length (const gchar *text, gint line_offset, guint len)
+{
+	const gchar *tab, *found_tab;
+	gint lo, cl, l, skip, sum_skip;
+
+	/* printf ("lo: %d o: %d t: '%s'\n", line_offset, len, text); */
+	l = 0;
+	lo = line_offset;
+	sum_skip = 0;
+	tab = text;
+	while (tab && (found_tab = strchr (tab, '\t')) && l < len) {
+		cl   = g_utf8_pointer_to_offset (tab, found_tab);
+		l   += cl;
+		if (l >= len)
+			break;
+		lo  += cl;
+		skip = 8 - (lo % 8);
+		tab  = found_tab + 1;
+		lo  += skip;
+		sum_skip += skip - 1;
+		l ++;
+	}
+
+	return len + sum_skip;
+}
+
+static guint
+get_line_length (HTMLObject *self, HTMLPainter *p, gint line_offset)
+{
+	return html_clueflow_tabs (HTML_CLUEFLOW (self->parent), p) || line_offset == -1
+		? html_text_text_line_length (HTML_TEXT (self)->text, line_offset, HTML_TEXT (self)->text_len)
+		: HTML_TEXT (self)->text_len;
+}
+
+inline gint
+html_text_get_line_offset (HTMLText *text, HTMLPainter *painter)
+{
+	return html_clueflow_tabs (HTML_CLUEFLOW (HTML_OBJECT (text)->parent), painter)
+		? html_clueflow_get_line_offset (HTML_CLUEFLOW (HTML_OBJECT (text)->parent), painter, HTML_OBJECT (text))
+		: -1;
+}
+
 static guint
 get_words (const gchar *s)
 {
@@ -538,33 +490,37 @@ get_words (const gchar *s)
 }
 
 static void
-calc_word_width (HTMLText *text, HTMLPainter *painter)
+calc_word_width (HTMLText *text, HTMLPainter *painter, gint line_offset)
 {
 	GtkHTMLFontStyle font_style;
 	gchar *begin, *end;
 	gint i;
 
 	text->words      = get_words (text->text);
+	if (text->word_width)
+		g_free (text->word_width);
 	text->word_width = g_new (guint, text->words);
 	font_style       = html_text_get_font_style (text);
 
-	begin = text->text;
-
+	begin            = text->text;
 	for (i = 0; i < text->words; i++) {
 		end   = strchr (begin + (i ? 1 : 0), ' ');
 		text->word_width [i] = (i ? text->word_width [i - 1] : 0)
 			+ html_painter_calc_text_width (painter,
-							begin, end ? unicode_index_to_offset (begin, end - begin)
-							: unicode_strlen (begin, -1), font_style, text->face);
+							begin, end ? g_utf8_pointer_to_offset (begin, end)
+							: g_utf8_strlen (begin, -1),
+							&line_offset, font_style, text->face);
 		begin = end;
 	}
+
+	HTML_OBJECT (text)->change &= ~HTML_CHANGE_WORD_WIDTH;
 }
 
 void
 html_text_request_word_width (HTMLText *text, HTMLPainter *painter)
 {
-	if (!text->word_width)
-		calc_word_width (text, painter);
+	if (!text->word_width || HTML_OBJECT (text)->change & HTML_CHANGE_WORD_WIDTH)
+		calc_word_width (text, painter, html_text_get_line_offset (text, painter));
 }
 
 static gint
@@ -578,10 +534,22 @@ calc_preferred_width (HTMLObject *self,
 	html_text_request_word_width (text, painter);
 
 	return MAX (1, text->word_width [text->words - 1]);
+}
 
-	/* return html_painter_calc_text_width (painter,
-					     text->text, text->text_len,
-					     font_style, text->face); */
+static void
+remove_text_slaves (HTMLObject *self)
+{
+	HTMLObject *next_obj;
+
+	/* Remove existing slaves */
+	next_obj = self->next;
+	while (next_obj != NULL
+	       && (HTML_OBJECT_TYPE (next_obj) == HTML_TYPE_TEXTSLAVE)) {
+		self->next = next_obj->next;
+		html_clue_remove (HTML_CLUE (next_obj->parent), next_obj);
+		html_object_destroy (next_obj);
+		next_obj = self->next;
+	}
 }
 
 static HTMLFitType
@@ -592,24 +560,15 @@ fit_line (HTMLObject *o,
 	  gint widthLeft) 
 {
 	HTMLText *text; 
-	HTMLObject *next_obj;
 	HTMLObject *text_slave;
 
 	text = HTML_TEXT (o);
 
 	if (o->flags & HTML_OBJECT_FLAG_NEWLINE)
 		return HTML_FIT_COMPLETE;
-	
-	/* Remove existing slaves */
-	next_obj = o->next;
-	while (next_obj != NULL
-	       && (HTML_OBJECT_TYPE (next_obj) == HTML_TYPE_TEXTSLAVE)) {
-		o->next = next_obj->next;
-		html_clue_remove (HTML_CLUE (next_obj->parent), next_obj);
-		html_object_destroy (next_obj);
-		next_obj = o->next;
-	}
-	
+
+	remove_text_slaves (o);
+
 	/* Turn all text over to our slaves */
 	text_slave = html_text_slave_new (text, 0, HTML_TEXT (text)->text_len, 0);
 	html_clue_append_after (HTML_CLUE (o->parent), text_slave, o);
@@ -761,70 +720,99 @@ get_length (HTMLObject *self)
 
 /* #define DEBUG_NBSP */
 
-static gboolean
-is_convert_nbsp_needed (const gchar *s, gint *delta_out)
+static inline gboolean
+check_last_white (gboolean rv, gint white_space, gunichar last_white, gint *delta_out)
 {
-	unicode_char_t uc;
-	gboolean rv = FALSE;
-	gboolean in_white_space;
-	const gchar *p, *op;
-
-	*delta_out = 0;
-
-	op = p = s;
-	in_white_space = FALSE;
-	while (*p && (p = unicode_get_utf8 (p, &uc))) {
-		if (uc == ENTITY_NBSP) {
-			if (!in_white_space) {
-				(*delta_out) --;
-				rv = TRUE;
-			}
-			in_white_space = TRUE;
-		} else if (uc == ' ') {
-			if (in_white_space) {
-				(*delta_out) ++;
-				rv = TRUE;
-			}
-			in_white_space = TRUE;
-		} else
-			in_white_space = FALSE;
-
-		op = p;
+	if (white_space > 0 && last_white == ENTITY_NBSP) {
+		(*delta_out) --;
+		rv = TRUE;
 	}
 
 	return rv;
 }
 
-static void
-convert_nbsp (gchar *fill, const gchar *p)
+static inline gboolean
+check_prev_white (gboolean rv, gint white_space, gunichar last_white, gint *delta_out)
 {
-	gboolean in_white_space;
-	unicode_char_t uc;
-	const gchar *op;
+	if (white_space > 0 && last_white == ' ') {
+		(*delta_out) ++;
+		rv = TRUE;
+	}
 
-#ifdef DEBUG_NBSP
-	printf ("convert_nbsp: %s --> ", s);
-#endif
-	op = p;
-	in_white_space = FALSE;
+	return rv;
+}
 
-	while (*p && (p = unicode_get_utf8 (p, &uc))) {
+static gboolean
+is_convert_nbsp_needed (const gchar *s, gint *delta_out)
+{
+	gunichar uc, last_white = 0;
+	gboolean rv = FALSE;
+	gint white_space;
+	const gchar *p, *op;
+
+	*delta_out = 0;
+
+	op = p = s;
+	white_space = 0;
+	while (*p && (p = e_unicode_get_utf8 (p, &uc))) {
 		if (uc == ENTITY_NBSP || uc == ' ') {
-			if (in_white_space) {
+			rv = check_prev_white (rv, white_space, last_white, delta_out);
+			white_space ++;
+			last_white = uc;
+		} else {
+			rv = check_last_white (rv, white_space, last_white, delta_out);
+			white_space = 0;
+		}
+		op = p;
+	}
+	rv = check_last_white (rv, white_space, last_white, delta_out);
+
+	return rv;
+}
+
+static inline void
+write_prev_white_space (gint white_space, gchar **fill)
+{
+			if (white_space > 0) {
 #ifdef DEBUG_NBSP
 				printf ("&nbsp;");
 #endif
-				*fill = 0xc2; fill ++;
-				*fill = 0xa0; fill ++;
-			} else {
-#ifdef DEBUG_NBSP
-				printf (" ");
-#endif
-				*fill = ' '; fill++;
+				**fill = 0xc2; (*fill) ++;
+				**fill = 0xa0; (*fill) ++;
 			}
-			in_white_space = TRUE;
+}
+
+static inline void
+write_last_white_space (gint white_space, gchar **fill)
+{
+	if (white_space > 0) {
+#ifdef DEBUG_NBSP
+		printf (" ");
+#endif
+		**fill = ' '; (*fill) ++;
+	}
+}
+
+static void
+convert_nbsp (gchar *fill, const gchar *p)
+{
+	gint white_space;
+	gunichar uc;
+	const gchar *op;
+
+#ifdef DEBUG_NBSP
+	printf ("convert_nbsp: %s --> \"", p);
+#endif
+	op = p;
+	white_space = 0;
+
+	while (*p && (p = e_unicode_get_utf8 (p, &uc))) {
+		if (uc == ENTITY_NBSP || uc == ' ') {
+			write_prev_white_space (white_space, &fill);
+			white_space ++;
 		} else {
-			in_white_space = FALSE;
+			write_last_white_space (white_space, &fill);
+			white_space = 0;
 #ifdef DEBUG_NBSP
 			printf ("*");
 #endif
@@ -834,10 +822,11 @@ convert_nbsp (gchar *fill, const gchar *p)
 		op = p;
 	}
 
+	write_last_white_space (white_space, &fill);
 	*fill = 0;
 
 #ifdef DEBUG_NBSP
-	printf ("\n");
+	printf ("\"\n");
 #endif
 }
 
@@ -989,7 +978,7 @@ set_font_style (HTMLText *text,
 
 	text->font_style = style;
 
-	html_object_change_set (HTML_OBJECT (text), HTML_CHANGE_ALL);
+	html_object_change_set (HTML_OBJECT (text), HTML_CHANGE_ALL_CALC);
 
 	if (engine != NULL) {
 		html_object_relayout (HTML_OBJECT (text)->parent, engine, HTML_OBJECT (text));
@@ -1112,7 +1101,7 @@ set_link (HTMLObject *self, HTMLColor *color, const gchar *url, const gchar *tar
 {
 	HTMLText *text = HTML_TEXT (self);
 
-	return html_link_text_new_with_len (text->text, text->text_len, text->font_style, color, url, target);
+	return url ? html_link_text_new_with_len (text->text, text->text_len, text->font_style, color, url, target) : NULL;
 }
 
 static void
@@ -1151,8 +1140,8 @@ get_cursor (HTMLObject *self,
 
 	slave = self->next;
 	if (slave == NULL || HTML_OBJECT_TYPE (slave) != HTML_TYPE_TEXTSLAVE) {
-		ascent = 0;
-		descent = 0;
+		ascent = self->ascent;
+		descent = self->descent;
 	} else {
 		ascent = slave->ascent;
 		descent = slave->descent;
@@ -1186,13 +1175,18 @@ get_cursor_base (HTMLObject *self,
 			if (offset != slave->posStart) {
 				HTMLText *text;
 				GtkHTMLFontStyle font_style;
+				gint line_offset;
 
 				text = HTML_TEXT (self);
 
 				font_style = html_text_get_font_style (text);
+				line_offset = html_text_slave_get_line_offset (slave,
+									       html_text_get_line_offset (HTML_TEXT (self),
+													  painter),
+									       slave->posStart, painter);
 				*x += html_painter_calc_text_width (painter,
 								    html_text_get_text (text, slave->posStart),
-								    offset - slave->posStart,
+								    offset - slave->posStart, &line_offset,
 								    font_style, text->face);
 			}
 
@@ -1202,6 +1196,7 @@ get_cursor_base (HTMLObject *self,
 
 	g_warning ("Getting cursor base for an HTMLText with no slaves -- %p\n",
 		   self);
+	html_object_calc_abs_position (self, x, y);
 }
 
 void
@@ -1240,6 +1235,7 @@ html_text_class_init (HTMLTextClass *klass,
 	object_class->check_point = check_point;
 	object_class->select_range = select_range;
 	object_class->get_length = get_length;
+	object_class->get_line_length = get_line_length;
 	object_class->set_link = set_link;
 	object_class->append_selection_string = append_selection_string;
 
@@ -1257,7 +1253,11 @@ html_text_class_init (HTMLTextClass *klass,
 inline static gint
 text_len (const gchar *str, gint len)
 {
-	return len == -1 ? unicode_strlen (str, -1) : len;
+	if (len == -1) {
+		return g_utf8_validate (str, -1, NULL) ? g_utf8_strlen (str, -1) : 0;
+	}
+
+	return len;
 }
 
 void
@@ -1272,7 +1272,7 @@ html_text_init (HTMLText *text,
 
 	html_object_init (HTML_OBJECT (text), HTML_OBJECT_CLASS (klass));
 
-	text->text          = len == -1 ? g_strdup (str) : g_strndup (str, unicode_offset_to_index (str, len));
+	text->text          = len == -1 ? g_strdup (str) : g_strndup (str, g_utf8_offset_to_pointer (str, len) - str);
 	text->text_len      = text_len (str, len);
 	text->font_style    = font_style;
 	text->face          = NULL;
@@ -1344,7 +1344,7 @@ html_text_set_font_style (HTMLText *text,
 {
 	g_return_if_fail (text != NULL);
 
-	return (* HT_CLASS (text)->set_font_style) (text, engine, style);
+	(* HT_CLASS (text)->set_font_style) (text, engine, style);
 }
 
 void
@@ -1355,7 +1355,7 @@ html_text_set_color (HTMLText *text,
 	g_return_if_fail (text != NULL);
 	g_return_if_fail (color != NULL);
 
-	return (* HT_CLASS (text)->set_color) (text, engine, color);
+	(* HT_CLASS (text)->set_color) (text, engine, color);
 }
 
 void
@@ -1466,25 +1466,25 @@ html_text_get_bytes (HTMLText *text)
 	return strlen (text->text);
 }
 
-guint
-html_text_get_index (HTMLText *text, guint offset)
-{
-	return unicode_offset_to_index (text->text, offset);
-}
-
-unicode_char_t
-html_text_get_char (HTMLText *text, guint offset)
-{
-	unicode_char_t uc;
-
-	unicode_get_utf8 (text->text + html_text_get_index (text, offset), &uc);
-	return uc;
-}
-
 gchar *
 html_text_get_text (HTMLText *text, guint offset)
 {
-	return text->text + html_text_get_index (text, offset);
+	return g_utf8_offset_to_pointer (text->text, offset);
+}
+
+guint
+html_text_get_index (HTMLText *text, guint offset)
+{
+	return html_text_get_text (text, offset) - text->text;
+}
+
+gunichar
+html_text_get_char (HTMLText *text, guint offset)
+{
+	gunichar uc;
+
+	uc = g_utf8_get_char (html_text_get_text (text, offset));
+	return uc;
 }
 
 /* magic links */
@@ -1499,9 +1499,9 @@ struct _HTMLMagicInsertMatch
 typedef struct _HTMLMagicInsertMatch HTMLMagicInsertMatch;
 
 static HTMLMagicInsertMatch mim [] = {
-	{ "(news|telnet|nttp|file|http|ftp|https)://[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(/[-a-z0-9_$.+!*(),;:@&=?/~#]*[^]'.}>) ,?!;:\"])?", NULL, NULL },
-	{ "www[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(/[-A-Za-z0-9_$.+!*(),;:@&=?/~#]*[^]'.}>) ,?!;:\"])?", NULL, "http://" },
-	{ "ftp[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(/[-A-Za-z0-9_$.+!*(),;:@&=?/~#]*[^]'.}>) ,?!;:\"])?", NULL, "ftp://" },
+	{ "(news|telnet|nttp|file|http|ftp|https)://([-a-z0-9]+(:[-a-z0-9]+)?@)?[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(/[-a-z0-9_$.+!*(),;:@%&=?/~#]*[^]'.}>\\) ,?!;:\"]?)?", NULL, NULL },
+	{ "www[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(/[-A-Za-z0-9_$.+!*(),;:@%&=?/~#]*[^]'.}>\\) ,?!;:\"]?)?", NULL, "http://" },
+	{ "ftp[-a-z0-9.]+[-a-z0-9](:[0-9]*)?(/[-A-Za-z0-9_$.+!*(),;:@%&=?/~#]*[^]'.}>\\) ,?!;:\"]?)?", NULL, "ftp://" },
 	{ "[-_a-z0-9.]+@[-_a-z0-9.]+", NULL, "mailto:" }
 };
 
@@ -1577,10 +1577,10 @@ html_text_magic_link (HTMLText *text, HTMLEngine *engine, guint offset)
 	while (offset < text->text_len && !rv) {
 		for (i=0; i<MIM_N; i++) {
 			if (mim [i].preg && !regexec (mim [i].preg, html_text_get_text (text, offset), 2, pmatch, 0)) {
-				gint o = html_text_get_text (text, offset) - text->text;
+				char *o = html_text_get_text (text, offset);
 				paste_link (engine, text,
-					    unicode_index_to_offset (text->text, pmatch [0].rm_so + o),
-					    unicode_index_to_offset (text->text, pmatch [0].rm_eo + o), mim [i].prefix);
+					    g_utf8_pointer_to_offset (text->text, o + pmatch [0].rm_so),
+					    g_utf8_pointer_to_offset (text->text, o + pmatch [0].rm_eo), mim [i].prefix);
 				rv = TRUE;
 				break;
 			}
